package com.redhat.eclipse.cdt.autotools;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.cdt.core.CCorePlugin;
import org.eclipse.cdt.core.CommandLauncher;
import org.eclipse.cdt.core.ConsoleOutputStream;
import org.eclipse.cdt.core.resources.IConsole;
import org.eclipse.cdt.make.core.IMakeTarget;
import org.eclipse.cdt.make.core.IMakeTargetManager;
import org.eclipse.cdt.make.core.makefile.IMakefile;
import org.eclipse.cdt.make.core.makefile.ITarget;
import org.eclipse.cdt.make.core.makefile.ITargetRule;
import org.eclipse.cdt.make.internal.core.makefile.posix.PosixMakefile;
import org.eclipse.cdt.managedbuilder.core.BuildException;
import org.eclipse.cdt.managedbuilder.core.IConfiguration;
import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
import org.eclipse.cdt.managedbuilder.core.IOption;
import org.eclipse.cdt.managedbuilder.core.ITool;
import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
import org.eclipse.cdt.managedbuilder.core.ManagedBuilderCorePlugin;
import org.eclipse.cdt.managedbuilder.envvar.IBuildEnvironmentVariable;
import org.eclipse.cdt.managedbuilder.makegen.IManagedBuilderMakefileGenerator;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;

public class MakeGenerator implements IManagedBuilderMakefileGenerator {

	public final String CONFIG_STATUS = "config.status"; //$NON-NLS-1$
	public final String AUTOGEN_SH = "autogen.sh";
	public final String CONFIGURE = "configure";
	public final String SETTINGS_FILE_NAME = ".cdtconfigure";

	private IProject project;

	private IProgressMonitor monitor;

	private String buildDir;
	private String srcDir;

	private IManagedBuildInfo info;

	private Vector invalidDirList;

	public void generateDependencies() throws CoreException {
		// TODO Auto-generated method stub

	}

	public MultiStatus generateMakefiles(IResourceDelta delta)
			throws CoreException {
		return regenerateMakefiles();
	}

	public void initialize(IProject project, IManagedBuildInfo info,
			IProgressMonitor monitor) {
		this.project = project;
		this.info = info;
		this.monitor = monitor;
		ITool tool = info.getToolFromOutputExtension("status");
		IOption[] options = tool.getOptions();
		for (int i = 0; i < options.length; ++i) {
			String id = options[i].getId();
			if (id.indexOf("builddir") > 0) {
				this.buildDir = (String) options[i].getValue();
			} else if (id.indexOf("configdir") > 0) {
				this.srcDir = (String) options[i].getValue();
			}
		}
	}

	public boolean isGeneratedResource(IResource resource) {
		// TODO Auto-generated method stub
		return false;
	}

	public void regenerateDependencies(boolean force) throws CoreException {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc) Check whether the build has been cancelled. Cancellation
	 * requests propagated to the caller by throwing <code>OperationCanceledException</code>.
	 * 
	 * @see org.eclipse.core.runtime.OperationCanceledException#OperationCanceledException()
	 */
	protected void checkCancel() {
		if (monitor != null && monitor.isCanceled()) {
			throw new OperationCanceledException();
		}
	}

	/*
	 * (non-Javadoc) Return or create the makefile needed for the build. If we
	 * are creating the resource, set the derived bit to true so the CM system
	 * ignores the contents. If the resource exists, respect the existing
	 * derived setting.
	 * 
	 * @param makefilePath @return IFile
	 */
	private IFile createFile(IPath makefilePath) throws CoreException {
		// Create or get the handle for the makefile
		IWorkspaceRoot root = CCorePlugin.getWorkspace().getRoot();
		IFile newFile = root.getFileForLocation(makefilePath);
		if (newFile == null) {
			newFile = root.getFile(makefilePath);
		}
		// Create the file if it does not exist
		ByteArrayInputStream contents = new ByteArrayInputStream(new byte[0]);
		try {
			newFile.create(contents, false, new SubProgressMonitor(monitor, 1));
			// Make sure the new file is marked as derived
			if (!newFile.isDerived()) {
				newFile.setDerived(true);
			}

		} catch (CoreException e) {
			// If the file already existed locally, just refresh to get contents
			if (e.getStatus().getCode() == IResourceStatus.PATH_OCCUPIED)
				newFile.refreshLocal(IResource.DEPTH_ZERO, null);
			else
				throw e;
		}

		return newFile;
	}

	/*
	 * (non-Javadoc) Return or create the folder needed for the build output. If
	 * we are creating the folder, set the derived bit to true so the CM system
	 * ignores the contents. If the resource exists, respect the existing
	 * derived setting.
	 * 
	 * @param string @return IPath
	 */
	private IPath createDirectory(String dirName) throws CoreException {
		// Create or get the handle for the build directory
		IFolder folder = project.getFolder(dirName);
		if (!folder.exists()) {
			// Make sure that parent folders exist
			IPath parentPath = (new Path(dirName)).removeLastSegments(1);
			// Assume that the parent exists if the path is empty
			if (!parentPath.isEmpty()) {
				IFolder parent = project.getFolder(parentPath);
				if (!parent.exists()) {
					createDirectory(parentPath.toString());
				}
			}

			// Now make the requested folder
			try {
				folder.create(true, true, null);
			} catch (CoreException e) {
				if (e.getStatus().getCode() == IResourceStatus.PATH_OCCUPIED)
					folder.refreshLocal(IResource.DEPTH_ZERO, null);
				else
					throw e;
			}

			// Make sure the folder is marked as derived so it is not added to
			// CM
			if (!folder.isDerived()) {
				folder.setDerived(true);
			}
		}

		return folder.getFullPath();
	}

	/*
	 * (non-Javadoc) Answers a Vector containing a list of directories that are
	 * invalid for the build for some reason. At the moment, the only reason a
	 * directory would not be considered for the build is if it contains a space
	 * in the relative path from the project root.
	 * 
	 * @return a a list of directories that are invalid for the build
	 */
	private Vector getInvalidDirList() {
		if (invalidDirList == null) {
			invalidDirList = new Vector();
		}
		return invalidDirList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.cdt.managedbuilder.makegen.IManagedBuilderMakefileGenerator#getBuildWorkingDir()
	 */
	public IPath getBuildWorkingDir() {
		return new Path(buildDir);
	}

	/*
	 * (non-Javadoc) Get the project's absolute path. @return IPath
	 */
	private IPath getProjectLocation() {
		return project.getLocation();
	}

	private String getAbsoluteDirectory(String dir) {
		IPath path = new Path(dir);
		if (!path.isAbsolute()) {
			IPath absPath = getProjectLocation().addTrailingSeparator().append(
					path);
			return absPath.toOSString();
		}
		return dir;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.cdt.managedbuilder.makegen.IManagedBuilderMakefileGenerator#getMakefileName()
	 */
	public String getMakefileName() {
		return new String(MAKEFILE_NAME);
	}

	public MultiStatus regenerateMakefiles() throws CoreException {
		MultiStatus status;
		boolean ok = true;
		boolean needFullConfigure = true;

		// See if the user has cancelled the build
		checkCancel();

		// Create the top-level directory for the build output
		createDirectory(buildDir);
		checkCancel();

		// // How did we do
		// if (!getInvalidDirList().isEmpty()) {
		// status = new MultiStatus (
		// ManagedBuilderCorePlugin.getUniqueIdentifier(),
		// IStatus.WARNING,
		// new String(),
		// null);
		// // Add a new status for each of the bad folders
		// iter = getInvalidDirList().iterator();
		// while (iter.hasNext()) {
		// status.add(new Status (
		// IStatus.WARNING,
		// ManagedBuilderCorePlugin.getUniqueIdentifier(),
		// SPACES_IN_PATH,
		// ((IContainer)iter.next()).getFullPath().toString(),
		// null));
		// }
		// } else {
		// status = new MultiStatus(
		// ManagedBuilderCorePlugin.getUniqueIdentifier(),
		// IStatus.OK,
		// new String(),
		// null);
		// }

		// Get the project and make sure there's a monitor to cancel the build
		IProject currentProject = project;
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}

		try {
			IWorkspace workspace = currentProject.getWorkspace();
			IWorkspaceRoot root = workspace.getRoot();
			// If a config.status file exists in the build directory, we call it
			// to
			// regenerate the makefile
			IPath configfile = getProjectLocation().append(buildDir).append(
					CONFIG_STATUS);
			IFile configStatus = root.getFileForLocation(configfile);
			IPath configSettingsPath = getProjectLocation().append(SETTINGS_FILE_NAME);
			IFile configSettings = root.getFileForLocation(configSettingsPath);
			String[] configArgs = getConfigArgs();

			// We need to figure out if the end-user has changed the configuration
			// settings.  In such a case, we need to reconfigure from scratch
			// regardless of whether config.status exists or not.
			// We figure this out by saving the configuration settings to
			// a special file and reading/comparing whenever we are asked to build.
			if (configSettings.exists()) {
				int i = 0;
				needFullConfigure = false;
				IPath settingsPath = project.getLocation().append(SETTINGS_FILE_NAME);
				try {
					File f = new File(settingsPath.toOSString());
					DataInputStream settings = new DataInputStream(
							new BufferedInputStream(new FileInputStream(f)));
					while (i < configArgs.length) {
						String s = settings.readUTF();
						if (!s.equals(configArgs[i])) {
							i = configArgs.length;
							needFullConfigure = true;
						}
						++i;
					}
					if (settings.available() > 0)
						needFullConfigure = true;
				} catch (EOFException e) {
					needFullConfigure = true;
				} catch (IOException e) {
					needFullConfigure = true;
				}
			}
			if (!needFullConfigure && configStatus != null && configStatus.exists()) {
				ok = runCommand(configfile, project.getLocation().append(
						buildDir), null, "Running config.status");
			}
			// Otherwise, look for configure and configure from scratch
			else if (configureExists()) {
				ok = runCommand(project.getLocation().append(
						info.getToolForConfiguration("status")), project
						.getLocation().append(buildDir),
						configArgs, "Generating Makefile");
				File makefileFile = project.getLocation().append(buildDir)
						.append("Makefile").toFile();
				addMakeTargetsToManager(makefileFile);
				// TODO: should we do something special if configure doesn't
				// return ok?
				saveConfigArgs(configArgs);
			}
			// If no configure, run autogen.sh
			else if (autogenExists()) {
				ok = runCommand(project.getLocation().append(AUTOGEN_SH),
						project.getLocation(), null,
						"Generating configure");
				// FIXME: does autogen run configure?
				ok = runCommand(project.getLocation().append(
						info.getToolForConfiguration("status")), project
						.getLocation().append(buildDir), null, "Generating Makefile");
				File makefileFile = project.getLocation().append(buildDir)
						.append("Makefile").toFile();
				addMakeTargetsToManager(makefileFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
			// forgetLastBuiltState();
			status = new MultiStatus(ManagedBuilderCorePlugin
					.getUniqueIdentifier(), IStatus.ERROR, new String(), null);
		} finally {
			// getGenerationProblems().clear();
			if (ok) {
				status = new MultiStatus(ManagedBuilderCorePlugin
						.getUniqueIdentifier(), IStatus.OK, new String(), null);
			} else {
				status = new MultiStatus(ManagedBuilderCorePlugin
						.getUniqueIdentifier(), IStatus.WARNING, new String(),
						null);
			}
		}
		return status;
	}

	private boolean configureExists() {
		IPath configCommand;
		if (srcDir.equals(""))
			configCommand = project.getLocation().append(CONFIGURE);
		else
			configCommand = project.getLocation().append(srcDir).append(
					CONFIGURE);
		return configCommand.toFile().exists();
	}

	private boolean autogenExists() {
		IPath autogenCommand;
		if (srcDir.equals(""))
			autogenCommand = project.getLocation().append(AUTOGEN_SH);
		else
			autogenCommand = project.getLocation().append(srcDir).append(
					AUTOGEN_SH);
		return autogenCommand.toFile().exists();
	}
	
	private void saveConfigArgs(String[] args) {
		IPath settingsPath = project.getLocation().append(SETTINGS_FILE_NAME);
		try {
			File f = new File(settingsPath.toOSString());
			DataOutputStream settings = new DataOutputStream(
					new BufferedOutputStream(new FileOutputStream(f)));
			for (int i = 0; i < args.length; ++i) {
				settings.writeUTF(args[i]);
			}
			settings.close();
		} catch (IOException e) {
			/* What should we do?  */
		}
	}
	
	private String[] getConfigArgs() throws BuildException {
		// Get the arguments to be passed to config from build model
		ITool tool = info.getToolFromOutputExtension("status");
		IOption[] options = tool.getOptions();
		ArrayList configArgs = new ArrayList();
		for (int i = 0; i < options.length; ++i) {
			if (options[i].getValueType() == IOption.STRING) {
				String value = (String) options[i].getValue();
				String id = options[i].getId();
				if (id.indexOf("configdir") > 0 || id.indexOf("builddir") > 0)
					continue;
				else if (id.indexOf("user") > 0) {
					// May be multiple user-specified options in which case we
					// need to split them up into individual options
					value = value.trim();
					boolean finished = false;
					int lastIndex = value.indexOf("--");
					if (lastIndex != -1) {
						while (!finished) {
							int index = value.indexOf("--",lastIndex+2);
							if (index != -1) {
								String previous = value.substring(lastIndex, index).trim();
								configArgs.add(previous);
								value = value.substring(index);
							} else {
								configArgs.add(value);
								finished = true;
							}
						}
					}
				}
				else if (value.trim().length() > 0) {
					String categoryId = options[i].getCategory().getId();
					if (categoryId.indexOf("directories") >= 0)
						value = getAbsoluteDirectory(value);
					String cmd = options[i].getCommand().concat(value);
					configArgs.add(cmd);
				}
			} else if (options[i].getValueType() == IOption.BOOLEAN) {
				Boolean value = (Boolean) options[i].getValue();
				if (value.booleanValue())
					configArgs.add(options[i].getCommand());
				else if (!options[i].getCommandFalse().equals(""))
					configArgs.add(options[i].getCommandFalse());
			}
		}
		return (String[]) configArgs.toArray(new String[configArgs.size()]);
	}

	private boolean runCommand(IPath commandPath, IPath runPath, String[] args,
			String jobDescription) throws BuildException, CoreException,
			NullPointerException, IOException {
		// TODO: Figure out what this next stuff is used for
		// //try to resolve the build macros in the builder command
		// try{
		// String resolved =
		// ManagedBuildManager.getBuildMacroProvider().resolveValueToMakefileFormat(
		// configCmd,
		// "", //$NON-NLS-1$
		// " ", //$NON-NLS-1$
		// IBuildMacroProvider.CONTEXT_CONFIGURATION,
		// info.getDefaultConfiguration());
		// if((resolved = resolved.trim()).length() > 0)
		// configCmd = resolved;
		// } catch (BuildMacroException e){
		// }

		boolean ok = false;
		
		String[] configTargets = args;
		if (configTargets == null)
			configTargets = new String[0];
		
		String[] msgs = new String[2];
		msgs[0] = commandPath.toString();
		msgs[1] = project.getName();
		monitor.subTask(AutotoolsPlugin.getFormattedString(
				"Makegenerator.make.message", msgs));


		// Get a build console for the project
		IConsole console = CCorePlugin.getDefault().getConsole("com.redhat.eclipse.cdt.autotools.configureConsole");
		IConfiguration cfg = null;
		ConsoleOutputStream consoleOutStream = null;
		StringBuffer buf = new StringBuffer();

		// Launch command - main invocation
		console.start(project);
		consoleOutStream = console.getOutputStream();
		String[] consoleHeader = new String[3];

		consoleHeader[0] = jobDescription;
		consoleHeader[1] = info.getConfigurationName();
		consoleHeader[2] = project.getName();
		buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$	//$NON-NLS-2$
		// buf.append(ManagedMakeMessages.getFormattedString(CONSOLE_HEADER,
		// consoleHeader));
		buf.append(AutotoolsPlugin
				.getResourceString("Makegenerator.configuring"));
		buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$	//$NON-NLS-2$
		buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$	//$NON-NLS-2$

		cfg = info.getDefaultConfiguration();
		if (!cfg.isSupported()) {
			// buf.append(ManagedMakeMessages.getFormattedString(WARNING_UNSUPPORTED_CONFIGURATION,new
			// String[] {cfg.getName(),cfg.getToolChain().getName()}));
			buf.append("Warning: unsupported configuration");
			buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$	//$NON-NLS-2$
			buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$	//$NON-NLS-2$
		}
		consoleOutStream.write(buf.toString().getBytes());
		consoleOutStream.flush();

		// Get a launcher for the config command
		String errMsg = null;
		CommandLauncher launcher = new CommandLauncher();
		// Set the environmennt
		IBuildEnvironmentVariable variables[] = ManagedBuildManager
				.getEnvironmentVariableProvider().getVariables(cfg, true, true);
		String[] env = null;
		ArrayList envList = new ArrayList();
		if (variables != null) {
			for (int i = 0; i < variables.length; i++) {
				envList.add(variables[i].getName()
						+ "=" + variables[i].getValue()); //$NON-NLS-1$
			}
			env = (String[]) envList.toArray(new String[envList.size()]);
		}

		// // Hook up an error parser manager
		// String[] errorParsers =
		// info.getDefaultConfiguration().getErrorParserList();
		// ErrorParserManager epm = new ErrorParserManager(project, topBuildDir,
		// this, errorParsers);
		// epm.setOutputStream(consoleOutStream);
		OutputStream stdout = consoleOutStream;
		OutputStream stderr = consoleOutStream;

		launcher.showCommand(true);
		Process proc = launcher.execute(commandPath, configTargets, env,
				runPath);
		if (proc != null) {
			try {
				// Close the input of the process since we will never write to
				// it
				proc.getOutputStream().close();
			} catch (IOException e) {
			}

			if (launcher.waitAndRead(stdout, stderr, new SubProgressMonitor(
					monitor, IProgressMonitor.UNKNOWN)) != CommandLauncher.OK) {
				errMsg = launcher.getErrorMessage();
			}

			// Force a resync of the projects without allowing the user to
			// cancel.
			// This is probably unkind, but short of this there is no way to
			// ensure
			// the UI is up-to-date with the build results
			// monitor.subTask(ManagedMakeMessages
			// .getResourceString(REFRESH));
			monitor.subTask("Refresh");
			try {
				project.refreshLocal(IResource.DEPTH_INFINITE, null);
			} catch (CoreException e) {
				monitor.subTask(AutotoolsPlugin
						.getResourceString("MakeGenerator.refresh.error"));
			}
		} else {
			errMsg = launcher.getErrorMessage();
		}

		// Report either the success or failure of our mission
		buf = new StringBuffer();
		if (errMsg != null && errMsg.length() > 0) {
			String errorDesc = AutotoolsPlugin
					.getResourceString("MakeGenerator.generation.error");
			buf.append(errorDesc);
			buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
			buf.append("(").append(errMsg).append(")"); //$NON-NLS-1$ //$NON-NLS-2$
			ok = false;
		} else {
			// Report a successful build
			String successMsg = AutotoolsPlugin
					.getResourceString("MakeGenerator.makefile.built");
			buf.append(successMsg);
			buf.append(System.getProperty("line.separator", "\n")); //$NON-NLS-1$//$NON-NLS-2$
			ok = true;
		}

		// Write message on the console
		consoleOutStream.write(buf.toString().getBytes());
		consoleOutStream.flush();

		// // Generate any error markers that the build has discovered
		// monitor.subTask(ManagedMakeMessages
		// .getResourceString(MARKERS));
		// epm.reportProblems();
		consoleOutStream.close();
		return ok;
	}

	private void addMakeTargetsToManager(File makefileFile) {
		FileReader fileReader;
		IMakeTargetManager makeTargetManager = AutotoolsPlugin.getDefault()
				.getTargetManager();
		try {
			fileReader = new FileReader(makefileFile);
			IMakefile makefile = new PosixMakefile();
			makefile.parse(makefileFile.toString(), fileReader);
			ITargetRule[] targets = makefile.getTargetRules();
			ITarget target = null;
			for (int i = 0; i < targets.length; i++) {
				target = targets[i].getTarget();
				String targetName = target.toString();
				if (!isValidTarget(targetName, makeTargetManager))
					continue;
				try {
					IMakeTarget makeTarget = makeTargetManager.createTarget(
							project, targetName, "genmakebuilder");
					makeTarget.setStopOnError(true);
					makeTarget.setRunAllBuilders(false);
					makeTarget.setUseDefaultBuildCmd(true);

					makeTarget.setBuildAttribute(IMakeTarget.BUILD_TARGET,
							targetName);
					
					if (makeTarget != null
							&& !makeTargetManager.targetExists(makeTarget)) {
						makeTargetManager.addTarget(project, makeTarget);
					} else {
						// FIXME:  is there anything else we want to do?
						// if (!makeTarget.getName().equals(targetName)) {
						//     makeTargetManager.renameTarget(target, getTargetName());
						// }
					}
				} catch (CoreException e) {
					// Duplicate target.  Ignore.
				}
			}
		} catch (FileNotFoundException e) {
			// No Makefile found ... something went wrong in the generation
			return;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private boolean isValidTarget(String targetName, IMakeTargetManager makeTargetManager) {
		try {
			return !(targetName.endsWith("-am")
					|| (makeTargetManager.findTarget(project, targetName) != null)
					|| targetName.endsWith("PROGRAMS")
					|| targetName.endsWith("-generic")
					|| (targetName.indexOf('$') >= 0)
					|| (targetName.charAt(0) == '.')
					|| targetName.equals(targetName.toUpperCase()));
		} catch (CoreException e) {
			return false;
		}
	}

}