package com.redhat.eclipse.cdt.autotools.actions;

import org.eclipse.cdt.make.core.IMakeTarget;
import org.eclipse.cdt.make.ui.MakeLabelProvider;
import org.eclipse.cdt.make.ui.TargetBuild;
import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;

import com.redhat.eclipse.cdt.autotools.AutotoolsPlugin;
import com.redhat.eclipse.cdt.autotools.MakeGenerator;

/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class BuildSpecial extends AbstractTargetAction {
	/**
	 * The constructor.
	 */
	public BuildSpecial() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		IContainer container = getSelectedContainer();
		// FIXME, TODO:
		// make Makefile if necessary
		if (container != null) {
			ElementListSelectionDialog dialog = new ElementListSelectionDialog(getShell(), new MakeLabelProvider());
			try {
				// Get the targets listed for the top-level makefile.  If none
				// exist, we may not have a makefile yet so we should attempt to
				// generate the makefile via configuration.
				IMakeTarget[] targets = AutotoolsPlugin.getDefault().getTargetManager().getTargets(container);
				if (targets.length == 0) {
					IProject project = container.getProject();
					IManagedBuildInfo info = ManagedBuildManager.getBuildInfo(project);
					MakeGenerator generator = new MakeGenerator();
					generator.initialize(project, info, new NullProgressMonitor());
					generator.regenerateMakefiles();
				}
				targets = AutotoolsPlugin.getDefault().getTargetManager().getTargets(container);
				dialog.setElements(targets);
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			dialog.setMultipleSelection(false);
			dialog.setTitle(AutotoolsPlugin
					.getResourceString("BuildTargetDialog.title.makeTargetsFor")
					+ container.getFullPath().toString().substring(1));
			// This doesn't make sense any more now that we're only allowing
			// selection from our list, I don't think :) -- overholt 2006-05-19
//			String name = null;
//			try {
//				name = (String) container.getSessionProperty(new QualifiedName(
//						AutotoolsPlugin.getUniqueIdentifier(), "lastTarget")); //$NON-NLS-1$
//			} catch (CoreException e) {
//			}
//			if (name != null) {
//				IPath path = new Path(name);
//				name = path.segment(path.segmentCount() - 1);
//				IContainer targetContainer;
//				if (path.segmentCount() > 1) {
//					path = path.removeLastSegments(1);
//					targetContainer = (IContainer) container.findMember(path);
//				} else {
//					targetContainer = container;
//				}
//				IMakeTarget target = AutotoolsPlugin.getDefault()
//						.getTargetManager().findTarget(targetContainer, name);
//				if (target != null)
//					dialog.setTarget(target);
//			}
			if (dialog.open() == Window.OK) {
				Object[] selection = dialog.getResult();
				// There will only be one target since we set
				// multiple selection to false
				IMakeTarget target = (IMakeTarget) selection[0];
				IPath path =
					target.getContainer().getProjectRelativePath().removeFirstSegments(
							container.getProjectRelativePath().segmentCount());
				path = path.append(target.getName());
				try {
					container.setSessionProperty(
							new QualifiedName(AutotoolsPlugin
									.getUniqueIdentifier(), "lastTarget"), //$NON-NLS-1$
							path.toString());
				} catch (CoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				TargetBuild.buildTargets(getShell(), new IMakeTarget[] { target });
			}
		}
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

}